@@ -60,6 +60,7 @@ void
 totem_sidebar_toggle (Totem *totem, gboolean state)
 {
 	GtkAction *action;
+	GtkWidget *box, *arrow;
 
 	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (totem->sidebar)) == state)
 		return;
@@ -74,6 +75,10 @@ totem_sidebar_toggle (Totem *totem, gboolean state)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), state);
 	totem_signal_unblock_by_data (G_OBJECT (action), totem);
 
+	box = GTK_WIDGET (gtk_builder_get_object (totem->xml, "tmw_sidebar_button_hbox"));
+	arrow = g_object_get_data (G_OBJECT (box), "arrow");
+	gtk_arrow_set (GTK_ARROW (arrow), state ? GTK_ARROW_LEFT : GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
+
 	totem->sidebar_shown = state;
 	cb_resize(totem);
 }
@@ -90,6 +95,41 @@ totem_sidebar_is_visible (Totem *totem)
 	return totem->sidebar_shown;
 }
 
+static gboolean
+has_popup (void)
+{
+	GList *list, *l;
+	gboolean retval = FALSE;
+
+	list = gtk_window_list_toplevels ();
+	for (l = list; l != NULL; l = l->next) {
+		GtkWindow *window = GTK_WINDOW (l->data);
+		if (GTK_WIDGET_VISIBLE (window) && gtk_window_get_window_type (window) == GTK_WINDOW_POPUP) {
+			retval = TRUE;
+			break;
+		}
+	}
+	g_list_free (list);
+	return retval;
+}
+
+gboolean
+totem_sidebar_is_focused (Totem *totem, gboolean *handles_kbd)
+{
+	GtkWidget *focused;
+
+	if (handles_kbd != NULL)
+		*handles_kbd = has_popup ();
+
+	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
+	if (focused != NULL && gtk_widget_is_ancestor
+			(focused, GTK_WIDGET (totem->sidebar)) != FALSE) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 void
 totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 {
@@ -124,8 +164,15 @@ totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 	gtk_widget_show_all (totem->sidebar);
 	gtk_widget_realize (totem->sidebar);
 
-	if (!visible)
+	if (!visible) {
 		gtk_widget_hide (totem->sidebar);
+	} else {
+		GtkWidget *box, *arrow;
+
+		box = GTK_WIDGET (gtk_builder_get_object (totem->xml, "tmw_sidebar_button_hbox"));
+		arrow = g_object_get_data (G_OBJECT (box), "arrow");
+		gtk_arrow_set (GTK_ARROW (arrow), GTK_ARROW_LEFT, GTK_SHADOW_NONE);
+	}
 }
 
 char *
